function [img_2x2dec,img_2x2lp_rec,img05_ilp]=IdealVsNonidealSampling_CRC;

% IdealVsNonidealSampling
% Comparing image sampling using pre and post-filters with rect- and sinc-functionPSF
% img05_ilp - test image reconstructed using sinc-function PSF of pre- and  post-filters
% img_2x2lp_rec - test image reconstructed using rect-function PSF of pre- and  post-filters
% img_2x2dec - test image reconstructed from decimated image using
% post-filter rect-function PSF
% Copyright L. Yaroslavsky (www.eng.tau.ac.il/~yaro; yaro@eng.tau.ac.il)
% Usage:   [img_2x2dec,img_2x2lp,img_2x2lp_rec]=IdealVsNonidealSampling_CRC;

img=load_img; % load test image text256
m=menu('Sampling rate','2','3','4');
m=m+1;
img_2x2dec=kron(img(1:m:end,1:m:end),ones(m,m));
img05_ilp=low_pass(img,1/m,1/m,1,1,0);
img_2x2lp=conv2(img,ones(m)/m^2,'same');
img_2x2lp_sampl=img_2x2lp(1:m:end,1:m:end);
img_2x2lp_rec=kron(img_2x2lp_sampl,ones(m,m));
figure_n(4);
figure(1);
imshow(img,[]);
title('Initial image','FontWeight','Bold')
figure(2)
imshow(img_2x2dec,[])
title(['Image ',num2str(m),'x',num2str(m),' decimated'],'FontWeight','Bold')
figure(3);
imshow(img_2x2lp_rec,[])
title(['Image ',num2str(m),'x',num2str(m),' LP-pre-filtered,decimated and reconstructed'],'FontWeight','Bold')
figure(4);
imshow(img05_ilp,[])
title(['Image ',num2str(m),'x',num2str(m),' ideally sampled and reconstructed'],'FontWeight','Bold')

%---------------------------------------------------------
function OUTIMG=low_pass(INPIMG,BWX,BWY,SmPar,QuantFlag,DisplFlag);

% Low pass filtering of an image
% BWX and BWY are fractions of bandwidth along X and Y to be left
% SmPar - size of the spectrum mask smoothing window (odd number) to reduce oscillations in output image
% Recommended values for SmPar: 3,5
% Processing result is quantized if QuantlFlag==1
% Processing result is displayed if DisplFlag==1
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il).
% Last modified 15.11.2004 
% Last modified 25.2.2005 to work with color images
% Call OUTIMG=low_pass(INPIMG,BWX,BWY,SmPar,QuantFlag,DisplFlag);

if DisplFlag==1,figure_n(3);figure(1);imshow(INPIMG/255);title('Lowpass.m: Input image');drawnow;end

[SzX SzY SzZ]=size(INPIMG);
OUTIMG=zeros(size(INPIMG));
BX=round(SzX*(1-BWX)/2);
BY=round(SzY*(1-BWY)/2);
SmPar2=SmPar^2;
mask=ones(SzX, SzY);
mask(SzX/2+1-BX:SzX/2+1+BX,:)=zeros(2*BX+1,SzY);
mask(:,SzY/2+1-BY:SzY/2+1+BY)=zeros(SzX,2*BY+1);
% mask smoothing
mask=fftshift(mask);
for k=1:9,mask=(conv2((mask),ones(SmPar)/SmPar2,'same'));end
mask=fftshift(mask);

% Display
if DisplFlag==1,figure(2);display1(fftshift(mask));title('mask');drawnow,end

for z=1:SzZ,
    sp=fft2(INPIMG(:,:,z));
    OUTIMG(:,:,z)=real(ifft2(sp.*mask));
    if QuantFlag==1,OUTIMG=round(srezka(OUTIMG));end
    if DisplFlag==1,figure(3);imshow(OUTIMG/255);title(['Lowpass.m: Output image, BWX=',num2str(BWX),'; BWY=',num2str(BWY)]);drawnow,end
end
%--------------------------------------------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% figure('Position',[HorPosition VerPosition HorSize VertSize]);
% Coordinate origin - left bottom corner of the screen
% Version July 15, 2001; Modified Oct. 2008, Tampere
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
% close all;

switch N;
case 1, close all
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
%    figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
    figure('Position',[80 VerSz-80-V_Sz V_Sz V_Sz]); colormap(gray(256))

case 2, close all 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-200);
%    figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
    figure('Position',[30 VerSz-200-H_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-200-H_Sz H_Sz V_Sz]);colormap(gray(256))

case 3,close all 
%    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-220)/2;
%    figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))

case 4,close all; 
%    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-220)/2;
%    figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
case 5,
   close all
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-220)/2;
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[85+2*H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   close all
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-220)/2;
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[85+2*H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+2*H_Sz VerSz-180-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end

%---------------------------------------------------
function OUTIMG=load_img;

% A browser for loading images in all formats
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTIMG=load_img;

dr=pwd;
cd c:\ ;
[FILENAME, PATHNAME] = uigetfile('*.*', 'LOAD IMAGE in *.tif format');

eval(['cd ',dr]);

OUTIMG=imread([PATHNAME FILENAME]);
OUTIMG=double(OUTIMG);
