function quantization_demo;

% Demonstration of quantization artifacts 
% and optimal P-th low quantization
% Modes: 
% "False contours"         - observe appearence in a selected image of false
%                            contours due to coarse quantization
% "Sensitivity threshold"  - detect on display test targets of different
%                            size and contrast and determine the detection
%                            threshold contrast as a function of the target size
% "Quantization range"     - Imaqe quantization to different number of quantization levels 
%                            in the selected dynamic range (in units of the image standard deviation) 
% "Quantization of spectra"- P-th low quantization of real and imaginary parts of
%                            image DFT spectra and optimization of
%                            quantization parameter P
% 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Call quantization_demo_CRC;

ss=1;
while ss==1;
m=menu('Select','False contours','Sensitivity threshold','Quantization range','Quantization of spectra');
switch m,
case 1
  	gg=menu('Select image','Karelia','Jerus','Chest Xray','Angiogram','MRI brain','Lena');
   switch gg,
   case 1, load karelia_;brain=karelia_;
   case 2, load jerus; brain=jerus;
   case 3, load chest; brain=chest;
   case 4, load ango;brain=ango;
   case 5, load brain;
   case 6, load lena;brain=lena;
   end
   s=1;
   sp=abs(fftshift(fft2(brain)));
   while s==1,
      fig_n(3);
      m1=menu('Select number of quantization levels','4','8','16','32','64');
      OUTIMG=quant_ize(brain,2^(m1+1));
      sp_q=abs(fftshift(fft2(OUTIMG)));
      figure(1);
      display_1(brain);
      title('Initial image','FontWeight','Bold');
      figure(2);
      display_1(OUTIMG);
      title(['Quantized image. Q=',num2str(2^(m1+1))],'FontWeight','Bold');
      figure(3);
      display_1(brain-OUTIMG);
      title('Quantization error','FontWeight','Bold');
      er=brain-OUTIMG;
      er=er-mean(er(:));
      sp_er=abs(fftshift(fft2(er)));
      s=menu('Repeat?','Yes','No');
   end
case 2
   sz=[1,2,3,5,7,11];
   g=zeros(1,6);
   fig_n(1)
   for k=1:6,
        target=zeros(256);target(64+fix(128*rand),64+fix(128*rand))=1; 
        target=conv2(target,ones(sz(k)),'same');
        bg=128*ones(256);bg(1,1)=0;bg(255,255)=255;
        sss=2;
%         g(k)=1;
%         figure(1);
        while sss==2, 
            g(k)=g(k)+1;
            display_1(bg+g(k)*target);
            title(['Can you detect target? Target Sz=',num2str(sz(k)),'. Contrast g=',num2str(g(k))],'FontWeight','Bold');
            sss=menu('Target detected','Yes','No');
        end
   end
%    figure(2);
   plot(sz,g);
   axis([1 11 2 max(g)+1]);grid
   title('Sensitivity threshold','FontWeight','Bold');
   xlabel('Target size','FontWeight','Bold');
   ylabel('Threshold','FontWeight','Bold');
   set(gca,'FontWeight','Bold');
case 3
   gg=menu('Select image','Karelia','Jerus','Chest Xray','Angiogram','Mammogram','MRI','Brain','Lena');
   switch gg,
   case 1, load karelia;ango=karelia+2*rand(size(karelia));
   case 2, load jerus; ango=jerus+2*rand(size(jerus));
   case 3, load chest; ango=chest+2*rand(size(chest));
   case 4, load ango; ango=ango+2*rand(size(ango));
   case 5, load mam_delp_512;ango=mam_delp_512+2*rand(512);
   case 6, load MRI; ango=MRI+2*rand(size(MRI));
   case 7, load brain;ango=brain+2*rand(size(brain));
   case 8, load lena;ango=len+2*rand(size(len));
   end
   fig_n(4);
   figure(1);
   [ango,Mean,stdv]=disp_norm(ango,7);
   %imshow(ango,[]);axis off;
   title('Input image','FontWeight','Bold');
   h=img_histo(ango);
   figure(3);
   plot(h,'LineWidth',2);axis tight;grid
   title('Image histogram','FontWeight','Bold');
   xlabel('Gray level','FontWeight','Bold');
   set(gca,'FontWeight','Bold')
   s3=1;
   while s3==1;
   figure(2);
   mmm=menu('Select quantization range','3','4','5','6','7');
   [ango_q,Mean,stdv]=disp_norm(ango,mmm+2);
   title(['Image in the selected range; R=',num2str(mmm+2)],'FontWeight','Bold');
   figure(3);
   hh=zeros(1,256);hh(round(Mean))=max(h);
   hhh=zeros(1,256);hhh(ceil(Mean-(mmm+2)*stdv/2))=max(h);hhh(floor(Mean+(mmm+2)*stdv/2))=max(h);
   plot(0:255,h,0:255,hh,0:255,hhh,'LineWidth',2);
   axis tight;
   grid on
   xlabel('Gray level','FontWeight','Bold');
   title('Image histogram and the quantization range (green - mean, red -borders of the range)','FontWeight','Bold');
   set(gca,'FontWeight','Bold');
   figure(4);
   mmm1=menu('Select number of quantizatio levels','8','16','32','64','128');
   display_1(round((2^(mmm1+2)*ango_q/255)));
   title(['Quantized image; Q=',num2str(2^(mmm1+1))],'FontWeight','Bold');
   s3=menu('Repeat','Yes','No');
   end
case 4,
   gg=menu('Select image','Jerus','Chest Xray','Angiogram','Mammogram','MRI','brain','Lena');
   switch gg,
        case 1, load jerus; INPIMG=jerus;
        case 2, load chest; INPIMG=chest;
        case 3, load ango; INPIMG=ango;
        case 4, load mam_delp_512;INPIMG=mam_delp_512;
        case 5, load MRI; INPIMG=MRI;
        case 6, load brain;INPIMG=brain;
        case 7, load lena;INPIMG=len;
   end
   s4=1;
   while s4==1,
   m1=menu('Select number of quantization levels','5','9','17','33','65');
   [OUTIMG,Stdev,P]=spectr_p_quant_optim(INPIMG,50,2^m1);
   s4=menu('Continue?','Yes','No');
   end
end
ss=menu('Repeat from the beginnig?','Y','No');
end

%----------------------------------------------------
function OUTIMG=quant_ize(INPIMG,Q);

% Image uniform  quantization in the range (INPIMGmin INPIMGmax)
% on Q levels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTIMG=quantize(INPIMG,Q);

inpmin=min(min(INPIMG));
inpmax=max(max(INPIMG));
OUTIMG=round((Q-1)*(INPIMG-inpmin)/(inpmax-inpmin));
OUTIMG=(inpmax-inpmin)*(OUTIMG)/(Q-1)+inpmin;
%------------------------------------------------------
   
 function display_1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display_1(INPIMG)

colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;
%-------------------------------------------------------
function [OUTIMG,Stdev,P]=spectr_p_quant_optim(INPIMG,Factor,Q);

% Optimization of P-th law quantization of INPIMG DFT spectrum
% In the range [-Factor*Spectrum_stdev,Factor*Spectrum_stdev]
% to 2Q+1 quantization levels
% Copyright L. Yaroslavsky(yaro@eng.tau.ac.il)
% Call [OUTIMG,stdev,P]=spectr_p_quant_optimiz(INPIMG);

INPSP=fft2(INPIMG);
Stdev=zeros(1,10);
P=[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1];
OUTIMG=[];
fig_n(4);
figure(1);
display_1(INPIMG);
title('Initial image','FontWeight','Bold')
figure(2);
for k=1:10,
   OUTSP=quan_spec_std(INPSP,Factor,Q,P(k));
   outimg=real(ifft2(OUTSP));
   display_1(outimg);
   title(['Reconstructed image, P=',num2str(P(k))],'FontWeight','Bold');
   drawnow
   [mn Stdev(k)]=std_2d(INPIMG-outimg);
   OUTIMG=[OUTIMG,outimg];
end
figure(3);
bar(P,Stdev,'b');
axis tight;grid;
title(['P-th law quantization error stdev; Q=',num2str(2*Q+1),'; DynRange=',num2str(Factor)],'FontWeight','Bold');
xlabel('Nonlinarity index P','FontWeight','Bold')
ylabel('Standard deviation of quantization error','FontWeight','Bold');
set(gca,'FontWeight','Bold');
k=find(Stdev==min(Stdev));
OUTSP=quan_spec_std(INPSP,Factor,Q,P(k));
outimg=real(ifft2(OUTSP));
figure(4);display_1(outimg);
title(['Reconstructed image for optimal P=',num2str(P(k))],'FontWeight','Bold');
%---------------------------------------------------------------

function fig_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call fig_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end

%------------------------------------------------------------------------------
function OUTSP=quan_spec_std(INPSP,Factor,Q,P);

% (P)th law quantization on 2Q+1 levels and (1/P)th law reconstruction
% of orthogonal components of DFT spectrum of an image
% In the range [-Factor*stdev,Factor*stdev]
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTSP=quanspec_std(INPSPECTR,Factor,Number of quantization levels,Power)

r=real(INPSP); im=imag(INPSP);
[mn,std_r]=std_2d(r);
[mn,std_im]=std_2d(im);
mx=Factor*sqrt(std_r^2+std_im^2);

absr=abs(r); absim=abs(im);
r_sr=(absr.*(absr<mx)+mx*(absr>=mx));
im_sr=(absim.*(absim<mx)+mx*(absim>=mx));

rq=mx*sign(r).*((round(Q*(r_sr/mx).^P))/Q).^(1/P);
imq=mx*sign(im).*((round(Q*(im_sr/mx).^P))/Q).^(1/P);
OUTSP=rq+i*imq;

%-----------------------------------------------------------
 
   function [mn,v]=std_2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std_2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));

%-------------------------------------------------------------------------------

function [dspl,Mean,stdv]=disp_norm(INPIMG,g)
% 2-D signal display with normalization by mean value and standard deviation
% g- is a parameter that defines ratio of the display range 0-255
% to the signal standard deviation 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call disp_norm(INPIMG,g)

[SzX SzY]=size(INPIMG);
Mean=mean(mean(INPIMG));		
stdv=sqrt(mean(mean((INPIMG-Mean).^2)))+eps;
dspl=round(255*(INPIMG-Mean)/(g*stdv))+128;
dspl=(dspl>=0).*dspl;
dspl=(dspl<=255).*dspl+255*ones(SzX,SzY).*(dspl>255);
image(dspl);axis image; axis off

%---------------------------------------------------------------------------
function OUTSP=quanspec_std(INPSP,Factor,Q,P);

% (P)th law quantization on 2Q+1 levels and (1/P)th law reconstruction
% of orthogonal components of DFT spectrum of an image
% In the range [-Factor*stdev,Factor*stdev]
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTSP=quanspec_std(INPSPECTR,Factor,Number of quantization levels,Power)

r=real(INPSP); im=imag(INPSP);
[mn,std_r]=std_2d(r);
[mn,std_im]=std_2d(im);
mx=Factor*sqrt(std_r^2+std_im^2);

absr=abs(r); absim=abs(im);
r_sr=(absr.*(absr<mx)+mx*(absr>=mx));
im_sr=(absim.*(absim<mx)+mx*(absim>=mx));

rq=mx*sign(r).*((round(Q*(r_sr/mx).^P))/Q).^(1/P);
imq=mx*sign(im).*((round(Q*(im_sr/mx).^P))/Q).^(1/P);
OUTSP=rq+i*imq;
%------------------------------------------------------------------------------------

function OUT=img_histo(INPIMG);

% Image histogram
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUT=imghisto(INPIMG);

[SzX SzY]=size(INPIMG);

histbuf=zeros(256,SzY);
for n=1:SzX
buf=INPIMG(n,:); buf=buf+256*([0:SzY-1]);
histbuf(buf+1)=histbuf(buf+1)+ones(1,SzY);
end

OUT=sum(histbuf'); 
   
   
   
